<?php
/**
 * Created by PhpStorm.
 * User: Zero
 * Date: 2021/06/24
 * Time: 15:59:06
 */

namespace App\Services;

use Meibuyu\Rpc\UploadOss\FileModel;
use Hyperf\Contract\ConfigInterface;
use League\Flysystem\Filesystem;
use Hyperf\Filesystem\FilesystemFactory;
use Hyperf\HttpMessage\Upload\UploadedFile;
use League\Flysystem\FileExistsException;
use Meibuyu\Micro\Exceptions\HttpResponseException;
use Psr\Container\ContainerInterface;

class UploadOssService
{

    /**
     * @var FilesystemFactory
     */
    private $factory;

    /**
     * @var ConfigInterface
     */
    private $config;

    /**
     * @var string
     */
    private $urlPrefix;

    /**
     * @var string
     */
    private $appDev;

    public static $options = [
        'maxSize' => 10 * 1024 * 1024, // 文件大小,10M
        'mime' => ['jpeg', 'png', 'gif', 'jpg', 'svg', 'txt', 'pdf', 'xlsx', 'xls', 'doc', 'docx', 'rar', 'zip', 'csv'], // 允许上传的文件类型
    ];

    public function __construct(ContainerInterface $container)
    {
        $this->config = $container->get(ConfigInterface::class);
        $this->factory = $container->get(FilesystemFactory::class);
        $ossConfig = $this->config->get('file.storage.oss');
        $this->urlPrefix = 'http://' . $ossConfig['bucket'] . '.' . $ossConfig['endpoint'] . '/';
        $this->appDev = $this->config->get('app_env');
    }

    /**
     * 上传图片
     * @param \Hyperf\HttpMessage\Upload\UploadedFile $image
     * @param string $path
     * @return array
     * @throws \Meibuyu\Micro\Exceptions\HttpResponseException
     * @author Zero
     */
    public function uploadImage(UploadedFile $image, string $path, string $userId)
    {
        $options = ['mime' => ['jpeg', 'png', 'gif', 'jpg', 'svg']];
        $filePath = 'meibuyu/' . $this->appDev . "/$path/images/" . today() . "/$userId/" . $image->getClientFilename();
        $this->upload($image, $filePath, $options);
        $model = new FileModel();
        $model = $model->newInstance([
            'type'  => 'image',
            'module'  => '',
            'user_id' => $userId,
            'size' => num_2_file_size($image->getSize()),
            'name' => $image->getClientFilename(),
            'path' => $this->urlPrefix . $filePath,
            'ext' => $image->getExtension(),
        ]);
        $model->save();
        return $model->toArray();
    }


    /**
     * 上传文件
     * @param \Hyperf\HttpMessage\Upload\UploadedFile $image
     * @param string $path
     * @return array
     * @throws \Meibuyu\Micro\Exceptions\HttpResponseException
     * @author Zero
     */
    public function uploadFile(UploadedFile $file, string $path, string $userId, array $options = [])
    {
        $filePath = 'meibuyu/' . $this->appDev . "/$path/images/" . today() . "/$userId/" . $file->getClientFilename();
        $this->upload($file, $filePath, $options);
        $model = new FileModel();
        $model = $model->newInstance([
            'type'  => 'file',
            'module'  => '',
            'user_id' => $userId,
            'size' => num_2_file_size($file->getSize()),
            'name' => $file->getClientFilename(),
            'path' => $this->urlPrefix . $filePath,
            'ext' => $file->getExtension(),
        ]);
        $model->save();
        return $model->toArray();
    }

    /**
     * @param \Hyperf\HttpMessage\Upload\UploadedFile $file
     * @param $path
     * @param array $options
     * @return bool
     * @throws \Meibuyu\Micro\Exceptions\HttpResponseException
     * @author Zero
     */
    public function upload(UploadedFile $file, string $path, array $options = [])
    {
        if ($file->isValid()) {
            $options = array_merge(self::$options, $options);
            $extension = strtolower($file->getExtension());
            // 通过扩展名判断类型
            if (!in_array($extension, $options['mime'])) {
                throw new HttpResponseException('文件类型不支持,目前只支持' . implode(',', $options['mime']));
            }
            // 判断文件大小
            if ($file->getSize() > $options['maxSize']) {
                throw new HttpResponseException('文件超出系统规定的大小,最大不能超过' . num_2_file_size($options['maxSize']));
            }
            try {
                $oss = $this->factory->get('oss');
                $stream = fopen($file->getRealPath(), 'r+');
                $res = $oss->writeStream($path, $stream);
                is_resource($stream) && fclose($stream);
                return $res;
            } catch (FileExistsException $e) {
                throw new HttpResponseException($e->getMessage());
            }
        } else {
            throw new HttpResponseException('文件无效');
        }
    }

}
